package nl.niftysystems.android.xboxlivestatswidget;

/*
 * Copyright (C) 2009 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

import nl.niftysystems.android.xboxlivestatswidget.ServiceHelper.ApiException;
import nl.niftysystems.android.xboxlivestatswidget.ServiceHelper.ParseException;
import android.app.PendingIntent;
import android.app.Service;
import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.IBinder;
import android.util.Log;
import android.widget.RemoteViews;

/**
 * XboxLiveStatsWidget, by Niftysystems 2009. 
 * 
 * This widget displays Xbox live gamertag stats. To build an update we
 * spawn a background {@link Service} to perfom the XML API queries to the
 * xbox live stats server.
 */
public class XboxLiveStatsWidget extends AppWidgetProvider {
	
	public static final String PREFS_NAME
	= "nl.niftysystems.android.XboxLiveStatsWidget";

    private static final String PREF_GAMERTAG = "gamertag_";
    
    public static void updateAppWidget(Context context,
			AppWidgetManager appWidgetManager, int mAppWidgetId,
			String gamertag) {
    	context.startService(new Intent(context, UpdateService.class));
	}
    
    public static class UpdateService extends Service {
        @Override
        public void onStart(Intent intent, int startId) {
        	Context context = this.getBaseContext();
        	AppWidgetManager gm = AppWidgetManager.getInstance(context);
            int[] appWidgetIds = gm.getAppWidgetIds(new ComponentName(context, XboxLiveStatsWidget.class));
          
            final int N = appWidgetIds.length;
            Log.e("Xbox", "appwidgetids.length=" + N);
            
            //Update all widgets
            for (int i=0; i<N; i++) {
            	// Build the widget update for today using the current appwidgetid
                RemoteViews updateViews = buildUpdate(this, appWidgetIds[i]);
                
                // Push update for this specific widget to the home screen
                AppWidgetManager manager = AppWidgetManager.getInstance(this);
                manager.updateAppWidget(appWidgetIds[i], updateViews);
            }
        }

        /**
         * Build a widget update to show the latest stats from the xbox api
         * Note: Will block until the online API returns.
         */
        public RemoteViews buildUpdate(Context context, int currentAppWidgetId) {
            // Get the resources, including some strings and context dependant variables
            Resources res = context.getResources();
            
            // Initialize the new RemoteViews and page content string array.
            RemoteViews updateViews = null;
            String[] pageContent = null;
            
            try {
            	// Prepare the user agent by setting some context dependant variables
                ServiceHelper.prepareUserAgent(context);
          
                //Load the gamertag preference for this instance of the widget
                SharedPreferences prefs = context.getSharedPreferences(PREFS_NAME, 0);
                String gamertag = prefs.getString(PREF_GAMERTAG + currentAppWidgetId, null);
                
                // Try querying the API for the xbox live stats
                pageContent = ServiceHelper.getPageContent(gamertag, res.getString(R.string.service_url));
            } catch (ApiException e) {
                Log.e("XboxLiveStatsWidget", "Couldn't contact API", e);
            } catch (ParseException e) {
                Log.e("XboxLiveStatsWidget", "Couldn't parse API response", e);
            }

            // Build an update that holds the updated widget contents 
            updateViews = new RemoteViews(context.getPackageName(), R.layout.widget_word);
            
            // Set the textual results (gamertag, info and gamerscore) from result array
            updateViews.setTextViewText(R.id.gamertag, pageContent[0]);
            updateViews.setTextViewText(R.id.gamerinfo, pageContent[1]);
            updateViews.setTextViewText(R.id.gamerscore, "Gamerpoints:" + pageContent[4] + "G");
            
            // Set online/offline presence icon
            if(pageContent[2].equalsIgnoreCase("true")) {
            	updateViews.setImageViewResource(R.id.onlinestatus, R.drawable.presence_online);
            } else {
            	updateViews.setImageViewResource(R.id.onlinestatus, R.drawable.presence_offline);
            }
            
            try {
                // Open a new URL and get the InputStream to load data from it.
                URL aURL = new URL(pageContent[3]);
                URLConnection conn = aURL.openConnection();
                conn.connect();
                InputStream is = conn.getInputStream();
                // Buffered is always good for a performance plus.
                BufferedInputStream bis = new BufferedInputStream(is);
                // Decode url-data to a bitmap.
                Bitmap bitmap = BitmapFactory.decodeStream(bis);
                bis.close();
                is.close();
                // Apply the Bitmap to the ImageView that will be returned.
                updateViews.setImageViewBitmap(R.id.gamertile, bitmap);
            } catch (IOException e) {
        	    updateViews.setImageViewResource(R.id.gamertile, R.drawable.icon);
                Log.e("Error", "Remote Image Exception", e);
           	}
          
            // When user clicks on widget, call the UpdateService class
            Intent defineIntent = new Intent(context, UpdateService.class);
            PendingIntent pendingIntent = PendingIntent.getService(context,
                    0 /* no requestCode */, defineIntent, 0 /* no flags */);
            updateViews.setOnClickPendingIntent(R.id.widget, pendingIntent);
            
            return updateViews;
        }
        
        @Override
        public IBinder onBind(Intent intent) {
            // We don't need to bind to this service
            return null;
        }
    }
}

